@@ -27,7 +27,7 @@ use Config::IniFiles;
 #inherits Lemonldap::NG::Common::Conf::Backends::SOAP
 #inherits Lemonldap::NG::Common::Conf::Backends::LDAP
 
-our $VERSION = '2.19.0';
+our $VERSION = '2.22.0';
 our $msg     = '';
 our $iniObj;
 
@@ -74,6 +74,8 @@ sub new {
             # Use local conf to get configStorage and localStorage
             my $localConfAccessPrms =
               $self->getLocalConf( CONFSECTION, $self->{confFile}, 0 );
+            $self->replacePlaceholders($localConfAccessPrms)
+              if $localConfAccessPrms->{useServerEnv};
             if ( defined $localConfAccessPrms ) {
                 %$self = ( %$self, %$localConfAccessPrms );
             }
@@ -125,10 +127,26 @@ sub saveConf {
 
     # If configuration was modified, return an error
     if ( not $args{force} ) {
-        return CONFIG_WAS_CHANGED
-          if ( $conf->{cfgNum} ne $last
-            || $args{cfgDate} && $args{cfgDate} ne $args{currentCfgDate} );
-        return DATABASE_LOCKED if ( $self->isLocked() or not $self->lock() );
+        if (   $conf->{cfgNum} ne $last
+            || $args{cfgDate} && $args{cfgDate} ne $args{currentCfgDate} )
+        {
+
+            if ( $conf->{cfgNum} ne $last ) {
+                $msg .=
+                  "Configuration $last was changed since $conf->{cfgNum}.\n";
+            }
+            if ( $args{cfgDate} ) {
+                $msg .=
+                    "Configuration date $args{currentCfgDate}"
+                  . " was changed since $args{cfgDate}.\n";
+            }
+
+            return CONFIG_WAS_CHANGED;
+        }
+        if ( $self->isLocked() or not $self->lock() ) {
+            $msg .= "Configuration is locked.\n";
+            return DATABASE_LOCKED;
+        }
     }
     $conf->{cfgNum} = $last + 1 unless ( $args{cfgNumFixed} );
     delete $conf->{cipher};
@@ -143,10 +161,17 @@ sub saveConf {
     }
 
     $msg .= "Configuration $conf->{cfgNum} stored.\n";
-    if ( $self->{refLocalStorage} ) {
-        $self->setDefault($conf);
-        $self->setValuesFromEnv($conf);
-        $self->compactConf($conf);
+    $self->setDefault($conf);
+    $self->setValuesFromEnv($conf);
+    $self->compactConf($conf);
+
+    if (    Lemonldap::NG::Handler::Main->can('tsv')
+        and Lemonldap::NG::Handler::Main->tsv->{msgBrokerWriter} )
+    {
+        Lemonldap::NG::Handler::Main->tsv->{msgBrokerWriter}
+          ->publish( $conf->{eventQueueName}, { action => 'newConf' } );
+    }
+    else {
         eval { Lemonldap::NG::Handler::Main->reload() };
     }
 
@@ -170,7 +195,11 @@ sub getConf {
     my $localPrm             = $args->{localPrm};
     my $noCache              = $args->{noCache};
     my $raw                  = $args->{raw};
-    my $allow_cache_for_root = $args->{allow_cache_for_root};
+    my $allow_cache_for_root = (
+          $self->{localStorageOptions}
+        ? $self->{localStorageOptions}->{allow_cache_for_root}
+        : 0
+    );
 
     # If running as UID=0, disabled the cache unless explicitely allowed
     if ( $> == 0 and !$allow_cache_for_root ) {
@@ -231,8 +260,9 @@ sub getConf {
 
                 # Store configuration in cache
                 $self->setLocalConf($r)
-                  if ( $self->{refLocalStorage}
-                    and not( $noCache == 1 or $raw ) );
+                  if (  $self->{refLocalStorage}
+                    and ( !$noCache or $noCache != 1 )
+                    and !$raw );
             }
         }
 
@@ -362,8 +392,11 @@ sub getLocalConf {
         foreach ( $cfg->Parameters(DEFAULTSECTION) ) {
             $r->{$_} = $cfg->val( DEFAULTSECTION, $_ );
             if ( $_ eq "require" ) {
-                eval { require $r->{$_} };
-                $msg .= "Error: $@" if ($@);
+
+                foreach my $require ( split /[,\s]+/, $r->{$_} ) {
+                    eval { require $require };
+                    $msg .= "Error: $@" if ($@);
+                }
             }
             if (   $r->{$_} =~ /^[{\[].*[}\]]$/
                 || $r->{$_} =~ /^sub\s*{.*}$/ )
@@ -377,27 +410,27 @@ sub getLocalConf {
         }
     }
 
-    # Stop if the requested section is the default section
-    return $r if ( $section eq DEFAULTSECTION );
+    # If we have an additional section to load
+    if ( $section ne DEFAULTSECTION and $cfg->SectionExists($section) ) {
 
-    # Check if requested section exists
-    return $r unless $cfg->SectionExists($section);
+        # Check if requested section exists
+        # Load section parameters
+        foreach ( $cfg->Parameters($section) ) {
+            $r->{$_} = $cfg->val( $section, $_ );
 
-    # Load section parameters
-    foreach ( $cfg->Parameters($section) ) {
-        $r->{$_} = $cfg->val( $section, $_ );
-
-        # Remove spaces before and after value (#1488)
-        $r->{$_} =~ s/^\s*(.+?)\s*/$1/;
-        if ( $r->{$_} =~ /^[{\[].*[}\]]$/ || $r->{$_} =~ /^sub\s*{.*}$/ ) {
-            eval "\$r->{$_} = $r->{$_}";
-            if ($@) {
-                $msg .= "Warn: error in file $file: $@.\n";
-                return $r;
+            # Remove spaces before and after value (#1488)
+            $r->{$_} =~ s/^\s*(.+?)\s*/$1/;
+            if ( $r->{$_} =~ /^[{\[].*[}\]]$/ || $r->{$_} =~ /^sub\s*{.*}$/ ) {
+                eval "\$r->{$_} = $r->{$_}";
+                if ($@) {
+                    $msg .= "Warn: error in file $file: $@.\n";
+                    return $r;
+                }
             }
         }
     }
 
+    $self->replacePlaceholders($r) if $self->{useServerEnv};
     return $r;
 }
 
@@ -533,7 +566,7 @@ sub logError {
 
 sub _substPlaceHolders {
     return $_[0] unless $_[0];
-    $_[0] =~ s/$PlaceHolderRe/$ENV{$1}/geo;
+    $_[0] =~ s/$PlaceHolderRe/$ENV{$1}/ge;
     return $_[0];
 }
 
@@ -544,7 +577,7 @@ sub replacePlaceholders {
     my ( $self, $conf ) = @_;
     if ( ref $conf eq 'HASH' ) {
         foreach my $key ( keys %$conf ) {
-            if ( $key =~ /$PlaceHolderRe/o ) {
+            if ( $key =~ /$PlaceHolderRe/ ) {
                 my $val = $conf->{$key};
                 delete $conf->{$key};
                 my $nk = _substPlaceHolders($key);
@@ -554,7 +587,7 @@ sub replacePlaceholders {
             if ( ref $conf->{$key} ) {
                 $self->replacePlaceholders( $conf->{$key} );
             }
-            elsif ( $conf->{$key} =~ /$PlaceHolderRe/o ) {
+            elsif ( $conf->{$key} =~ /$PlaceHolderRe/ ) {
                 $conf->{$key} = _substPlaceHolders( $conf->{$key} );
             }
         }
@@ -564,7 +597,7 @@ sub replacePlaceholders {
             if ( ref $conf->[$i] ) {
                 $self->replacePlaceholders( $conf->[$i] );
             }
-            elsif ( $conf->[$i] =~ /$PlaceHolderRe/o ) {
+            elsif ( $conf->[$i] =~ /$PlaceHolderRe/ ) {
                 $conf->[$i] = _substPlaceHolders( $conf->[$i] );
             }
         }
