@@ -6,7 +6,7 @@ use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '2.19.0';
+our $VERSION = '2.22.0';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
@@ -163,7 +163,8 @@ sub virtualHosts {
     }
 
     # Send setDefault for new vhosts
-    return $self->sendError( $req, 'setDefault', 200 ) if ( $vh =~ /^new__/ );
+    return $self->sendJSONresponse( $req, { error => 'setDefault' } )
+      if ( $vh =~ /^new__/ );
 
     # Reject unknown vhosts
     return $self->sendError( $req, "Unknown virtualhost ($vh)", 400 )
@@ -255,7 +256,7 @@ sub _samlMetaDataNodes {
     }
 
     # setDefault response for new partners
-    return $self->sendError( $req, 'setDefault', 200 )
+    return $self->sendJSONresponse( $req, { error => 'setDefault' } )
       if ( $partner =~ /^new__/ );
 
     # Reject unknown partners
@@ -309,6 +310,15 @@ sub _samlMetaDataNodes {
         return $self->sendJSONresponse( $req, { value => $value } );
     }
 
+    # Metadata URL
+    elsif ( $query =~ /^saml${type}MetaDataURL$/ ) {
+        my $value =
+          eval { $self->getConfKey( $req, $query )->{$partner}->{$query}; }
+          // undef;
+        return $self->sendError( $req, undef, 400 ) if ( $req->error );
+        return $self->sendJSONresponse( $req, { value => $value } );
+    }
+
     # These regexps are generated by jsongenerator.pl and stored in
     # Lemonldap::NG::Common::Conf::ReConstants
     elsif (
@@ -382,7 +392,7 @@ sub _oidcMetaDataNodes {
     }
 
     # setDefault response for new partners
-    return $self->sendError( $req, 'setDefault', 200 )
+    return $self->sendJSONresponse( $req, { error => 'setDefault' } )
       if ( $partner =~ /^new__/ );
 
     # Reject unknown partners
@@ -506,7 +516,7 @@ sub _casMetaDataNodes {
     }
 
     # setDefault response for new partners
-    return $self->sendError( $req, 'setDefault', 200 )
+    return $self->sendJSONresponse( $req, { error => 'setDefault' } )
       if ( $partner =~ /^new__/ );
 
     # Reject unknown partners
@@ -767,6 +777,39 @@ sub sfExtra {
     return $self->sendJSONresponse( $req, $res );
 }
 
+## Keys
+sub keyNodes {
+    my ( $self, $req, @path ) = @_;
+    {
+        local $" = ",";
+        $self->logger->debug("Keynode @path");
+    }
+
+    return $self->complexNodesRoot( $req, "keys", "keyNode" ) unless (@path);
+
+    my $keyId = shift @path;
+    my $query = shift @path;
+
+    unless ($query) {
+        return $self->sendError( $req,
+            "Bad request: keyNodes query must ask for a key", 400 );
+    }
+
+    # setDefault response for new keyIds
+    return $self->sendError( $req, 'setDefault', 200 )
+      if ( $keyId =~ /^new__/ );
+
+    # Reject unknown keyIds
+    return $self->sendError( $req, "Unknown key ID ($keyId)", 400 )
+      unless ( defined eval { $self->getConfKey( $req, 'keys' )->{$keyId}; } );
+
+    my ( $id, $resp ) = ( 1, [] );
+
+    my $value =
+      eval { $self->getConfKey( $req, "keys" )->{$keyId}->{$query}; } // undef;
+    return $self->sendJSONresponse( $req, { value => $value } );
+}
+
 # 33 - Root queries
 #      -----------
 
@@ -811,35 +854,61 @@ sub metadata {
             ],
         );
     }
-    elsif ( $req->params('oidcMetadata') ) {
-        my $c = $self->getConfKey( $req, 'cfgNum' );
+    elsif ( my $oidc = $req->params('oidcMetadata')
+        or $req->params('samlMetadata') )
+    {
+        my $c    = $self->getConfKey( $req, 'cfgNum' );
+        my $type = $oidc ? 'OIDC' : 'SAML';
         return $self->sendError( $req, undef, 400 ) if ( $req->error );
         if ( $self->can('userId') ) {
             $self->userLogger->notice( 'User '
                   . $self->userId($req)
-                  . ' ask for OIDC metadata '
+                  . " ask for $type metadata "
                   . $c );
         }
         else {
-            $self->logger->info("REST request to get OIDC metadata $c");
+            $self->logger->info("REST request to get $type metadata $c");
+        }
+        if ($oidc) {
+            require Lemonldap::NG::Common::OpenIDConnect::Metadata;
+            my $path = $self->currentConf->{issuerDBOpenIDConnectPath};
+            $path =~ s#^.*?(\w+).*?$#$1/#;
+            return $self->sendJSONresponse(
+                $req,
+                Lemonldap::NG::Common::OpenIDConnect::Metadata->metadataDoc(
+                    $self->currentConf->{oidcServiceMetaDataIssuer}
+                      || $self->p->portal,
+                    $self->currentConf,
+                    $path,
+                ),
+                pretty  => 1,
+                headers => [
+                    'content-Type'        => 'application/json',
+                    'Content-Disposition' =>
+                      "Attachment; filename=openid-configuration.json"
+                ],
+            );
+        }
+        else {
+            require Lemonldap::NG::Common::Conf::SAML::Metadata;
+            if ( my $metadata =
+                Lemonldap::NG::Common::Conf::SAML::Metadata->new() )
+            {
+                my $s =
+                  $metadata->serviceToXML(
+                    { %{ $self->currentConf }, portal => $self->p->portal },
+                    'all' );
+                return [
+                    200,
+                    [
+                        'Content-Type'   => 'application/xml',
+                        'Content-Length' => length($s),
+                    ],
+                    [$s]
+                ];
+            }
+            return $self->p->sendError( $req, 'Unable to build Metadata', 500 );
         }
-        require Lemonldap::NG::Common::OpenIDConnect::Metadata;
-        my $path = $self->currentConf->{issuerDBOpenIDConnectPath};
-        $path =~ s#^.*?(\w+).*?$#$1/#;
-        return $self->sendJSONresponse(
-            $req,
-            Lemonldap::NG::Common::OpenIDConnect::Metadata->metadataDoc(
-                $self->currentConf->{oidcServiceMetaDataIssuer}
-                  || $self->p->portal,
-                $self->currentConf,
-                $path,
-            ),
-            pretty  => 1,
-            headers => [
-                'content-Type'        => 'application/json',
-                'Content-Disposition' => "Attachment; filename=openid-configuration.json"
-            ],
-        );
     }
     else {
         my $res = {};
@@ -923,7 +992,7 @@ sub getKey {
 
     # When "hash"
     if ( $key =~ qr/^$simpleHashKeys$/o ) {
-        return $self->sendError( $req, 'setDefault', 200 )
+        return $self->sendJSONresponse( $req, { error => 'setDefault' } )
           unless defined($value);
 
         # If a hash key is asked return its value
@@ -961,7 +1030,7 @@ sub getKey {
     # When scalar
     return $self->sendError( $req, "Key $key is not a hash", 400 )
       if ($subkey);
-    return $self->sendError( $req, 'setDefault', 200 )
+    return $self->sendJSONresponse( $req, { error => 'setDefault' } )
       unless defined($value);
     return $self->sendJSONresponse( $req, { value => $value } );
 
