@@ -6,7 +6,7 @@ use Encode;
 use JSON;
 use Lemonldap::NG::Common::Conf::Constants;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.12';
 
 BEGIN {
     *Lemonldap::NG::Common::Conf::normalize      = \&normalize;
@@ -98,7 +98,26 @@ sub unserialize {
             unless ( utf8::is_utf8($v) ) {
                 $v = encode( 'UTF-8', $v );
             }
-            $conf->{$k} = ( $v =~ /./ ? eval { from_json($v) } : {} );
+            $conf->{$k} = (
+                $v =~ /./
+                ? eval { from_json( $v, { allow_nonref => 1 } ) }
+                : {}
+            );
+            if ($@) {
+                $Lemonldap::NG::Common::Conf::msg .=
+                  "Unable to decode $k, switching to old format.\n";
+                return $self->oldUnserialize($fields);
+            }
+        }
+        elsif ( $k =~ $arrayParameters ) {
+            unless ( utf8::is_utf8($v) ) {
+                $v = encode( 'UTF-8', $v );
+            }
+            $conf->{$k} = (
+                $v =~ /./
+                ? eval { from_json( $v, { allow_nonref => 1 } ) }
+                : {}
+            );
             if ($@) {
                 $Lemonldap::NG::Common::Conf::msg .=
                   "Unable to decode $k, switching to old format.\n";
@@ -127,7 +146,7 @@ sub oldUnserialize {
         if ( $k =~ $hashParameters and $v ||= {} and not ref($v) ) {
             $conf->{$k} = {};
 
-            # Value should be a Data::Dumper, else this is an old format
+            # Value should be a Data::Dumper, else this is an old-old format
             if ( defined($v) and $v !~ /^\$/ ) {
 
                 $Lemonldap::NG::Common::Conf::msg .=
